///////////////////  EXAMPLE 1 ///////////////
import fred SP500 CBBTCUSD VIXCLS, daterange(2017-01-03 2023-04-30) clear 
rename SP500     sp500
rename VIXCLS    vix
rename CBBTCUSD  bitcoin

describe
tsset daten,daily
list daten sp500 bitcoin vix  in 1/8,sep(4)

generate bdate=bofd("bcalstocks",daten)
format bdate %tbbcalstocks
tsset bdate
drop if bdate==.
list daten bdate sp500 bitcoin vix  in 1/8,sep(4)

///////////////////  EXAMPLE 2 ///////////////
import fred CHLSACRQISMEI, daterange(1990-01-01 2010-12-31) ///
	aggregate(quarterly) clear
	
rename CHLSACRQISMEI  cars_cl
describe

generate quarter = qofd(daten)
format quarter %tq

tsset quarter

list daten quarter cars_cl if tin(1994q1,1994q4)
list quarter cars_cl L.cars_cl L2.cars_cl F.cars_cl D.cars_cl ///
     if tin(1994q1,1994q4), noobs
tsline cars_cl,                                           ///
   title("Quarterly total passenger car registrations")    ///
   subtitle("Chile 1994q1-2010q4") ytitle(" ") xtitle(" ") ///
    note(Source: Federal Reserve databank)

tsline D.cars_cl,                                           ///
   title("Quarterly total passenger car registrations")    ///
   subtitle("Chile 1994q1-2010q4") ytitle(" ") xtitle(" ") ///
    note(Source: Federal Reserve databank)
	
corrgram D.cars_cl,lags(20)

arimasoc D.cars_cl

arima D.cars_cl,ar(1) nolog

///////////////////  EXAMPLE 3 ///////////////
import fred EXMXUS, daterange(2000-01-01 .) clear

generate month=mofd(daten)
label var month "numeric (monthly) date"
rename EXMXUS exchrate_mx

tsset month,monthly

list if tin(2023m1,2023m4),abbreviate(12)

tsline exchrate_mx, name(level,replace) ytitle(level) xtitle("")
tsline D.exchrate_mx, name(difference,replace)                ///
                      ytitle(First difference) xtitle("month")
graph combine level difference,rows(2)                        ///
     title("Mexican pesos to US dollar")                      ///
     note(Source: Federal Reserve databank) 

ac D.exchrate_mx,lags(25) ytitle(" ")  name(ac_dexchrate_mx,replace)   ///
   title("Autocorrelation function for D.exchrate_mx") ylabels(#4,angle(0))
pac D.exchrate_mx,lags(25) ytitle(" ") name(pac_dexchrate_mx,replace)   ///
    title("Partial autocorrelation function for D.exchrate_mx") ylabels(#4,angle(0))
	
graph combine ac_dexchrate_mx pac_dexchrate_mx,rows(2) 

arch D.exchrate_mx,arch(1) garch(1) ar(1/2) nolog vsquish

///////////////////  EXAMPLE 4 ///////////////

import fred SP500 CBBTCUSD VIXCLS, daterange(2017-01-02 2023-04-28) ///
   aggregate(weekly,avg) clear
rename SP500 sp500
rename CBBTCUSD bitcoin
rename VIXCLS vix

describe
tsset daten,delta(7 days)
tsline sp500,name(sp_levels,replace) ytitle("Levels")               ///
    title("S&P 500") ylabels(#3,angle(0)) xlabels(#3) xtitle("")
tsline D.sp500,name(sp_diff,replace) ytitle("First difference")      ///
    ylabels(#3,angle(0)) xlabels(#3) xtitle("")
tsline bitcoin,name(bitcoin_levels,replace) ytitle("Levels")         /// 
    title("Coinbase Bitcoin") ylabels(#3,angle(0)) xlabels(#3) xtitle("") 
tsline D.bitcoin,name(bitcoin_diff,replace) ytitle("First difference") ///
    ylabels(#3,angle(0)) xlabels(#3) xtitle("") 
graph combine sp_levels bitcoin_levels sp_diff bitcoin_diff,rows(2)  ///
    note(Source: Federal Reserve databank) 

tsline vix,name(vix_levels,replace) ytitle("Levels") 
tsline D.vix,name(vix_diff,replace) ytitle("First difference")   
graph combine vix_levels vix_diff,            ///
     title("Weekly CBOE Volatility Index: VIX")       ///
     note(Source: Federal Reserve databank) rows(2)

var D.sp500 D.bitcoin if tin(06mar2020,28apr2023), exog(D.vix) vsquish 
estimates store eq_var

irf create order1, step(10) set(myirf1,replace)
irf graph oirf,impulse(D.bitcoin) response(D.sp500) name(i_bit__r_sp,replace)
irf graph oirf,impulse(D.bitcoin) response(D.bitcoin) name(i_bit__r_bit,replace)
irf graph oirf,impulse(D.sp500) response(D.sp500) name(i_sp__r_sp,replace)
irf graph oirf,impulse(D.sp500) response(D.bitcoin) name(i_sp__r_bit,replace)

graph combine i_bit__r_sp i_bit__r_bit i_sp__r_sp i_sp__r_bit, ///
     title("Impulse-response functions")
	 
///////////////////  EXAMPLE 5 ///////////////
arima D.vix if tin(06mar2020,28apr2023), ar(1) nolog 
estimates store eq_ar1

tsappend, add(4)
forecast clear
forecast create myforecast
forecast estimates eq_var,names(dsp500 dbitcoin)
forecast estimates eq_ar1,names(dvix)
forecast identity sp500=dsp500+L.sp500
forecast identity bitcoin=dbitcoin+L.bitcoin
forecast identity vix=dvix+L.vix
forecast solve, begin(td(05may2023)) end(td(26may2023)) prefix(f_)

list daten sp500 f_sp500 bitcoin f_bitcoin vix f_vix ///
     if tin(07apr2023,26may2023), noobs

tsline f_sp500 sp500 if tin(07apr2023,26may2023),         ///
    lc(red blue) ylabels(#4,angle(0)) xtitle("")          ///
    title("S&P 500 weekly forecast Apr07-May26") name(sp_fcst,replace)
tsline f_bitcoin bitcoin if tin(07apr2023,26may2023),     ///
    lc(red blue) ylabels(#4,angle(0)) xtitle("")          ///
    title("Bitcoin weekly forecast Apr07-May26") name(b_fcst,replace)
graph combine sp_fcst b_fcst, rows(2)




///////////////////  EXAMPLE 6 ///////////////
import fred SP500 CBBTCUSD VIXCLS, daterange(2017-01-02 2023-04-28) ///
   aggregate(weekly,avg) clear
rename SP500 sp500
rename CBBTCUSD bitcoin
rename VIXCLS vix

tsset daten,delta(7 days)

quietly {
	arima D.vix if tin(06mar2020,28apr2023), ar(1) nolog
    tsappend,add(4)
    predict vix_hat, y dynamic(td(5may2023)) t0(td(28apr2023))
    replace vix=vix_hat if tin(5may2023,26may2023)
}

matrix b0 = J(1,2,0)
bayes,minnconjprior(mean(b0) selftight(1)) rseed(123) dryrun  ///
   saving(bvar_sim1,replace):                                 ///   
   var D.sp500 D.bitcoin if tin(06mar2020,28apr2023), exog(D.vix)  
bayes, minnconjprior(mean(b0) selftight(1)) rseed(123) noheader  ///
   nomodelsummary saving(bvar_sim1,replace):                    ///   
   var D.sp500 D.bitcoin if tin(06mar2020,28apr2023), exog(D.vix)  

bayesfcast compute bvar_, step(4) mcmcsaving(fcast_mcmc,replace)
/* Use mcmc simulations for the predicted outcome variables */
 use fcast_mcmc,clear
/* Rename to identify daily predictions */
  foreach var in D_sp500 D_bitcoin {
     rename `var'_23135 `var'_May5
     rename `var'_23142 `var'_May12
     rename `var'_23149 `var'_May19
 }
/* Separate events for t+1 to t+3 (May5, May12, May19) */
foreach var in D_sp500 D_bitcoin {
    foreach date in May5 May12 May19 {
       generate `var'_`date'_pos=cond(`var'_`date'>=0,1,0) 
   }
}
/* Combined events for one variable */
foreach var in D_sp500 D_bitcoin {
 gen `var'_pos=cond(`var'_May5>=0 & `var'_May12>=0 & `var'_May5>=0,1,0)
}
/* Combined events for both variables */
foreach date in May5 May12 May19 {
 gen sp_bc_`date'_pos=cond(D_sp500_`date'>=0 & D_bitcoin_`date'>=0,1,0)
}
generate sp_bc_pos=cond(D_sp500_May5>=0 & D_bitcoin_May5>=0   & ///
                        D_sp500_May12>=0 & D_bitcoin_May12>=0 & ///
                        D_sp500_May19>=0 & D_bitcoin_May19>=0,1,0)

***************************************
 collect clear
 local i=1
 foreach date in May5 May12 May19 {
      collect   sp500_pos = e(b)[1,2]  ///
             bitcoin_pos = e(b)[1,4]     ///
               sp_bc_pos = e(b)[1,6],    ///
         tags(date[`i']):                /// 
		 proportion D_sp500_`date'_pos   ///
                    D_bitcoin_`date'_pos ///
                    sp_bc_`date'_pos
      local ++i
 }
 
 collect sp500_pos=e(b)[1,2] bitcoin_pos=e(b)[1,4] sp_bc_pos=e(b)[1,6], ///
       tags(date[4]): proportion D_sp500_pos D_bitcoin_pos sp_bc_pos
	   
 quietly collect layout (result) (date)
 collect label levels date 1 "May5" 2 "May12" 3 "May19" 4 "May5-May19" 
 collect label levels result        ///
          sp500_pos  "Change in SP500 > 0"     ///
          bitcoin_pos "Change in Bitcoin > 0" ///
		  sp_bc_pos  "Change in SP500 & Bitcoin > 0"
 
 collect style header result, title(label)
 collect style cell result, nformat(%6.3f)
 collect style column, extraspace(1)
 
collect preview

///////////////////  EXAMPLE 7 ///////////////
import fred SP500 CBBTCUSD VIXCLS, daterange(2017-01-02 2023-04-28) ///
   aggregate(weekly,avg) clear
rename SP500 sp500
rename CBBTCUSD bitcoin
rename VIXCLS vix

tsset daten,delta(7 days)

lpirf D.sp500 D.bitcoin if tin(06mar2020,28apr2023),exog(D.vix)
irf set lpirf.irf, replace
irf create lp_model

irf graph dm, irf(lp_model) impulse(D.vix) response(D.bitcoin) ///
      yline(0) xlabel(0(2)8) name(irf_bitcoin,replace)         ///
      byopts(legend(off) note("") b1title(""))
irf graph dm, irf(lp_model) impulse(D.vix) response(D.sp500)   ///
      yline(0) xlabel(0(2)8) name(irf_sp500,replace)           ///
      byopts(legend(off) note(""))

graph combine irf_bitcoin irf_sp500,rows(2)                               ///
    title("Dynamic multiplier for one-unit shock in VIX")                 ///
	subtitle("Effect on Bitcoin and SP500")                               ///
	b2title("Graphs by irfname, impulse variable, and response variable", ///
	         size(small))

exit